import matplotlib.pyplot as plt
import numpy as np

plt.rcParams['savefig.format'] = 'svg'

def dqdt(qmLs, Dp, Aprox, Adist, Aopen, Aclose, Len):
    q = qmLs / 1e6
    rhob = 1050

    Avalve = q*0 + Aopen
    Avalve[q<0] = Aclose

    sign = q*0+1
    sign[q<0] = -1

    v1 = q / Aprox
    v2 = q / Avalve
    v3 = q / Adist
    L = 1.5 * rhob * (Len / Avalve + 0.5 * (1.0 / np.sqrt(Aprox) + 1.0 / np.sqrt(Adist)));
    DpB = (0.5 * rhob) * (v2**2 - v1**2)
    DpB_neg = (0.5 * rhob) * (-v2**2 - v3**2)
    DpB[q<0] = DpB_neg[q<0]
    return sign * (Dp - DpB) / L * 1e6




Dp_pos = 5
Dp_neg = -50
Aprox = 1e-3 # Lv max: 0.0016744083823976058
Adist = 5e-4 # SyArt max
Aopen = 5e-4 # LvSyArt
Aclose_fac_pos = 1e-9 # LvSyArt
Aclose_fac_neg = 0.01 # LvSyArt
Len = 0.4 # LvSyArt


def get_linestyle(value):
    line_style_changes = {'c':'k', 'ls':'-', 'lw':'1'}
    if value < 0:
        line_style_changes['c'] = 'b'
    if value > 0:
        line_style_changes['c'] = 'r'

    return line_style_changes

for i_flow in [0, 1]:
    fig = plt.figure(1+i_flow, clear=True, figsize=(12, 4))
    axs = []
    yl = [np.inf, 0]
    # reference max q = 0.00045962475019523704
    if i_flow==0:
        q = np.linspace(0, 1e6*8e-4, 101)
        Dp = Dp_pos * 133
        Aclose = Aclose_fac_pos * Aopen
    else:
        q = np.linspace(-25, -1e-27, 101)
        Dp = Dp_neg * 133
        Aclose = Aclose_fac_neg * Aopen

    for i_plot in range(5):
        ax = plt.subplot(1, 5, 1+i_plot)
        axs.append(ax)
        ax.plot(q, dqdt(q, Dp, Aprox, Adist, Aopen, Aclose, Len),
                  c='k', ls='-', lw='2')
        if i_flow==1:
            ax.plot(q, dqdt(q, Dp, Aprox, Adist, Aopen, 1e-9, Len),
                      c='k', ls='-', lw='2')

        ax.plot([np.min(q), np.max(q)], [0, 0],
                 c='k', lw=1, ls=':')

        if i_plot == 0:
            ax.set_title(r'Change in $\Delta p$')
            for dDp in np.linspace(-0.25*Dp, 0.25*Dp, 5):
                line_style_changes = get_linestyle(dDp)
                ax.plot(q, dqdt(q, Dp+dDp, Aprox, Adist, Aopen, Aclose, Len),
                          **line_style_changes)
        if i_plot == 1:
            plt.title(r'Change in $A_{prox}$')
            for dAprox in np.linspace(-4e-4, 4e-4, 5):
                line_style_changes = get_linestyle(dAprox)
                ax.plot(q, dqdt(q, Dp, Aprox+dAprox, Adist, Aopen, Aclose, Len),
                          **line_style_changes)

        if i_plot == 2 and i_flow == 0:
            plt.title(r'Change in $A_{open}$')
            for dAvalve in np.linspace(-4e-4, 4e-4, 6):
                line_style_changes = get_linestyle(dAvalve)
                ax.plot(q, dqdt(q, Dp, Aprox, Adist, Aopen+dAvalve, Aclose+dAvalve, Len),
                          **line_style_changes)

        if i_plot == 2 and i_flow == 1:
            plt.title(r'Change in $A_{close}$')
            for dAvalve in -np.linspace(0, 0.009999999999, 5) * Aopen:
                line_style_changes = get_linestyle(dAvalve)
                ax.plot(q, dqdt(q, Dp, Aprox, Adist, Aopen, Aclose+dAvalve, Len),
                          **line_style_changes)

        if i_plot == 3:
            plt.title(r'Change in $A_{distal}$')
            for dAdist in [-4.9999e-4, -4.9e-4, -4.5e-4, -4e-4, 0]:
                line_style_changes = get_linestyle(dAdist)
                ax.plot(q, dqdt(q, Dp, Aprox, Adist+dAdist, Aopen, Aclose, Len),
                          **line_style_changes)

        if i_plot == 4:
            plt.title(r'Change in $l$')
            for dLen in np.linspace(-0.2, 0.2, 5):
                line_style_changes = get_linestyle(dLen)
                ax.plot(q, dqdt(q, Dp, Aprox, Adist, Aopen, Aclose, Len+dLen),
                          **line_style_changes)

        # # style
        if i_plot > 0:
            ax.set_yticks([])

        if i_plot == 0:
            ax.set_ylabel(r'$\frac{dq}{dt}$ [$L/s^2$]')
        if i_plot == 2:
            ax.set_xlabel('q [L/s]')

        yl[0] = np.min([yl[0], ax.get_ylim()[0]])
        yl[1] = np.max([yl[1], ax.get_ylim()[1]])

        ax.spines[['right', 'top']].set_visible(False)

    if i_flow==0:
        yl = [-500, 1150]
        plt.suptitle(r'$\bf{Flow-time\ derivative\ for\ positive\ flow}$' \
                     '\n' + r'$\Delta p = '+str(Dp_pos)+' mmHg$ ',
                     fontsize=15)
    if i_flow==1:
        yl = [-100, 100]
        plt.suptitle(r'$\bf{Flow-time\ derivative\ for\ positive\ flow}$' \
                     '\n' + r'$\Delta p = '+str(Dp_neg)+r' mmHg,A_{open} = 5cm^2$ ',

                     fontsize=15)
    for ax in axs:
        ax.set_ylim(yl)


    plt.tight_layout()
plt.draw()
